;;; Spanish translations of DEFINTERNATIONALed values.
;;; Bruno Haible, Carlos Linares

(in-package "LISP")

(export 'ESPAŅOL)

(deflanguage ESPAŅOL)

(in-package "SYSTEM")

(deflocalized y-or-n ESPAŅOL '((#\N) . (#\S #\Y)))
(deflocalized yes-or-no ESPAŅOL '(("no") . ("si")))

